@@ -1,5 +1,8 @@
 #!./miniruby
 
+# Used by the "make install" target to install Ruby.
+# See common.mk for more details.
+
 begin
   load "./rbconfig.rb"
 rescue LoadError
@@ -26,7 +29,7 @@ rescue LoadError
 end
 
 STDOUT.sync = true
-File.umask(0222)
+File.umask(077)
 
 def parse_args(argv = ARGV)
   $mantype = 'doc'
@@ -281,9 +284,8 @@ def with_destdir(dir)
 end
 
 def without_destdir(dir)
-  return dir if !$destdir or $destdir.empty? or !dir.start_with?($destdir)
-  dir = dir.sub(/\A\w:/, '') if File::PATH_SEPARATOR == ';'
-  dir[$destdir.size..-1]
+  return dir if !$destdir or $destdir.empty?
+  dir.start_with?($destdir) ? dir[$destdir.size..-1] : dir
 end
 
 def prepare(mesg, basedir, subdirs=nil)
@@ -357,7 +359,7 @@ install?(:local, :arch, :lib) do
   prepare "base libraries", libdir
 
   install lib, libdir, :mode => $prog_mode, :strip => $strip unless lib == arc
-  install arc, libdir, :mode => $data_mode
+  install arc, libdir, :mode => $data_mode unless CONFIG["INSTALL_STATIC_LIBRARY"] == "no"
   if dll == lib and dll != arc
     for link in CONFIG["LIBRUBY_ALIASES"].split
       ln_sf(dll, File.join(libdir, link))
@@ -387,6 +389,15 @@ install?(:ext, :arch, :'ext-arch') do
   install_recursive("#{$extout}/#{CONFIG['arch']}", archlibdir, :no_install => noinst, :mode => $prog_mode, :strip => $strip)
   prepare "extension objects", sitearchlibdir
   prepare "extension objects", vendorarchlibdir
+  if extso = File.read("exts.mk")[/^EXTSO[ \t]*=[ \t]*((?:.*\\\n)*.*)/, 1] and
+    !(extso = extso.gsub(/\\\n/, '').split).empty?
+    libpathenv = CONFIG["LIBPATHENV"]
+    dest = CONFIG[!libpathenv || libpathenv == "PATH" ? "bindir" : "libdir"]
+    prepare "external libraries", dest
+    for file in extso
+      install file, dest, :mode => $prog_mode
+    end
+  end
 end
 install?(:ext, :arch, :hdr, :'arch-hdr') do
   prepare "extension headers", archhdrdir
@@ -416,16 +427,16 @@ install?(:doc, :capi) do
   install_recursive "doc/capi", docdir+"/capi", :mode => $data_mode
 end
 
-if load_relative
+if load_relative or /\s/ =~ bindir
   PROLOG_SCRIPT = <<EOS
 #!/bin/sh\n# -*- ruby -*-
-bindir="${0%/*}"
+bindir="#{load_relative ? '${0%/*}' : bindir.gsub(/\"/, '\\\\"')}"
 EOS
   if CONFIG["LIBRUBY_RELATIVE"] != 'yes' and libpathenv = CONFIG["LIBPATHENV"]
     pathsep = File::PATH_SEPARATOR
     PROLOG_SCRIPT << <<EOS
-prefix="${bindir%/bin}"
-export #{libpathenv}="$prefix/lib${#{libpathenv}:+#{pathsep}$#{libpathenv}}"
+libdir="#{load_relative ? '${bindir%/bin}/lib' : libdir.gsub(/\"/, '\\\\"')}"
+export #{libpathenv}="$libdir${#{libpathenv}:+#{pathsep}$#{libpathenv}}"
 EOS
   end
   PROLOG_SCRIPT << %Q[exec "$bindir/#{ruby_install_name}" -x "$0" "$@"\n]
@@ -637,21 +648,6 @@ module RbInstall
         string.sub(/\A#{Regexp.escape(prefix)}/, "")
       end
     end
-
-    class Reader < Struct.new(:src)
-      def gemspec
-        @gemspec ||= begin
-          spec = Gem::Specification.load(src) || raise("invalid spec in #{src}")
-          file_collector = FileCollector.new(File.dirname(src))
-          spec.files = file_collector.collect
-          spec
-        end
-      end
-
-      def spec_source
-        @gemspec.to_ruby
-      end
-    end
   end
 
   class UnpackedInstaller < Gem::Installer
@@ -676,6 +672,9 @@ module RbInstall
 
     def write_cache_file
     end
+
+    def build_extensions
+    end
   end
 end
 
@@ -704,25 +703,25 @@ install?(:ext, :comm, :gem) do
   default_spec_dir = "#{spec_dir}/default"
   makedirs(default_spec_dir)
 
-  gems = {}
-
-  Dir.glob(srcdir+"/{lib,ext}/**/*.gemspec").each do |src|
-    specgen   = RbInstall::Specs::Reader.new(src)
-    gems[specgen.gemspec.name] ||= specgen
-  end
-
-  gems.sort.each do |name, specgen|
-    gemspec   = specgen.gemspec
+  gems = Dir.glob(srcdir+"/{lib,ext}/**/*.gemspec").map {|src|
+    spec = Gem::Specification.load(src) || raise("invalid spec in #{src}")
+    file_collector = RbInstall::Specs::FileCollector.new(File.dirname(src))
+    files = file_collector.collect
+    next if files.empty?
+    spec.files = files
+    spec
+  }
+  gems.compact.sort_by(&:name).each do |gemspec|
     full_name = "#{gemspec.name}-#{gemspec.version}"
 
     puts "#{" "*30}#{gemspec.name} #{gemspec.version}"
     gemspec_path = File.join(default_spec_dir, "#{full_name}.gemspec")
     open_for_install(gemspec_path, $data_mode) do
-      specgen.spec_source
+      gemspec.to_ruby
     end
 
     unless gemspec.executables.empty? then
-      bin_dir = File.join(gem_dir, 'gems', full_name, 'bin')
+      bin_dir = File.join(gem_dir, 'gems', full_name, gemspec.bindir)
       makedirs(bin_dir)
 
       execs = gemspec.executables.map {|exec| File.join(srcdir, 'bin', exec)}
@@ -748,11 +747,26 @@ install?(:ext, :comm, :gem) do
     :wrappers => true,
     :format_executable => true,
   }
-  Gem::Specification.each_spec([srcdir+'/gems/*']) do |spec|
+  gem_ext_dir = "#$extout/gems/#{CONFIG['arch']}"
+  extensions_dir = Gem::StubSpecification.gemspec_stub("", gem_dir, gem_dir).extensions_dir
+  Gem::Specification.each_gemspec([srcdir+'/gems/*']) do |path|
+    dir = File.dirname(path)
+    spec = Dir.chdir(dir) {
+      Gem::Specification.load(File.basename(path))
+    }
+    next unless spec.platform == Gem::Platform::RUBY
+    next unless spec.full_name == path[srcdir.size..-1][/\A\/gems\/([^\/]+)/, 1]
+    spec.extension_dir = "#{extensions_dir}/#{spec.full_name}"
+    if File.directory?(ext = "#{gem_ext_dir}/#{spec.full_name}")
+      spec.extensions[0] ||= "-"
+    end
     ins = RbInstall::UnpackedInstaller.new(spec, options)
     puts "#{" "*30}#{spec.name} #{spec.version}"
     ins.install
     File.chmod($data_mode, File.join(install_dir, "specifications", "#{spec.full_name}.gemspec"))
+    unless spec.extensions.empty?
+      install_recursive(ext, spec.extension_dir)
+    end
     installed_gems[spec.full_name] = true
   end
   installed_gems, gems = Dir.glob(srcdir+'/gems/*.gem').partition {|gem| installed_gems.key?(File.basename(gem, '.gem'))}
@@ -762,12 +776,14 @@ install?(:ext, :comm, :gem) do
   next if gems.empty?
   if defined?(Zlib)
     Gem.instance_variable_set(:@ruby, with_destdir(File.join(bindir, ruby_install_name)))
+    silent = Gem::SilentUI.new
     gems.each do |gem|
+      inst = Gem::Installer.new(gem, options)
+      inst.spec.extension_dir = with_destdir(inst.spec.extension_dir)
       begin
-        File.umask(022)
-        Gem.install(gem, Gem::Requirement.default, options)
-      ensure
-        File.umask(0222)
+        Gem::DefaultUserInteraction.use_ui(silent) {inst.install}
+      rescue Gem::InstallError => e
+        next
       end
       gemname = File.basename(gem)
       puts "#{" "*30}#{gemname}"
